/*

Copyright (C) 2000, 2001, 2002 Christian Kreibich <christian@whoop.org>.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies of the Software and its documentation and acknowledgment shall be
given in the documentation and software packages that this Software was
used.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/
#ifndef __nd_udp_h
#define __nd_udp_h

#include <netdude/nd.h>
#include <netdude/nd_gettext.h>
#include <netdude/nd_packet.h>
#include <netdude/nd_trace.h>
#include <netdude/nd_protocol.h>

#include <netinet/in.h>
#include <netinet/udp.h>


/* These are the implementations of ND_Protocol's callbacks: */
GtkWidget *nd_udp_create_gui(ND_Trace *trace, ND_ProtoInfo *pinf);
void       nd_udp_set_gui(const ND_Packet *packet, ND_ProtoInfo *pinf);
void       nd_udp_init_packet(ND_Packet *packet, guchar *data, guchar *data_end);
gboolean   nd_udp_header_complete(const ND_Packet *packet, guint nesting);
gboolean   nd_udp_datagram_complete(const ND_Packet *packet, guint nesting);
gboolean   nd_udp_fix_packet(ND_Packet *packet, int index);

/* Generic accessor function for the protocol implementation: */
ND_Protocol *nd_udp_get(void);

guint16    nd_udp_checksum(const ND_Packet *packet);
gboolean   nd_udp_csum_correct(const ND_Packet *packet, guint16 *correct_sum);

/* Helper functions to set a trace's GUI table to a packet's content: */
void       nd_udp_set_gui_sport(ND_ProtoInfo *pinf, struct udphdr *udphdr);
void       nd_udp_set_gui_dport(ND_ProtoInfo *pinf, struct udphdr *udphdr);
void       nd_udp_set_gui_len(ND_ProtoInfo *pinf, struct udphdr *udphdr);
void       nd_udp_set_gui_csum(ND_ProtoInfo *pinf, struct udphdr *udphdr, const ND_Packet *packet);

#endif
